%% MODIFIED YAROSLAVSKY ALGORITHM
% THOR ARNE S. BRANDSVOLL
%
%EXAMPLE OF HOW TO USE:
%
%DENOISING AVI FILE: READ A FRAME AT THE TIME, DENOISE, AND SAVE RESULT IN
%NEW AVI FILE. MUST USE A GLOBAL VARIABLE "result".
%
%t = cputime; 
%
% readerobj = VideoReader('c:\test.avi');
%
% numFrames = get(readerobj, 'numberOfFrames');
%
% writerobj = VideoWriter('c:\result.avi','Uncompressed AVI');
% writerobj.FrameRate = 60;
% open(writerobj);
%
% global result;
%
% for i = 1:numFrames
%   
%    frame = double(read(readerobj,i));
%    result = yaroslavsky(frame);
%    writeVideo(writerobj,uint8(result(:,:,:,1)));
%
% end
%
% close(writerobj);   
%
% total_runtime = cputime-t


function  res = yaroslavsky(in);

t = cputime; 
 
% check if input is single frame or video, and number of planes
video_size = size(in);
if size(video_size,2) > 2;
    number_of_planes = video_size(3);
else
    number_of_planes = 1;
end

if size(video_size,2) > 3;
    number_of_frames = video_size(4);
else
    number_of_frames = 1;
end

rows = video_size(1);
columns = video_size(2);

threshold1 = 5;
threshold2 = 10;
threshold3 = 25;

weight_ct_px    = 12;
weight1         = 12;
weight2         = 8;
weight3         = 4;

% Original Yaroslavsky filter have equal weights 
% weight_ct_px    = 1;
% weight1         = 1;
% weight2         = 1;
% weight3         = 1;

% if input is several frames, save result for temporal filtering locally 
if number_of_frames > 1
    y_t1 = in(:,:,:,1);    %initial y(t-1)
    res(:,:,:,1)=in(:,:,:,1); 
    
    for k = 2:number_of_frames
        
        y_t = in(:,:,:,k);
        
        for j = 1:number_of_planes
            
              
        %Pad image
        y_t_pad = padarray(y_t(:,:,j,1), [1 1], 'symmetric');
        y_t1_pad = padarray(y_t1(:,:,j,1), [1 1], 'symmetric');
        
        %Matrix with all pixels shifted one position
        y_t_left_shift = y_t_pad(2:rows+1,1:columns);
        y_t_right_shift = y_t_pad(2:rows+1,3:columns+2);
        y_t_top_shift = y_t_pad(1:rows,2:columns+1);
        y_t_bottom_shift = y_t_pad(3:rows+2,2:columns+1);
        
        y_t1_left_shift = y_t1_pad(2:rows+1,1:columns);
        y_t1_right_shift = y_t1_pad(2:rows+1,3:columns+2);
        y_t1_top_shift = y_t1_pad(1:rows,2:columns+1);
        y_t1_bottom_shift = y_t1_pad(3:rows+2,2:columns+1);
      
%COMMENTED CODE IS FOR A LARGER NEIGHBORHOOD
%         %corner pixels
%         y_t_left_top_shift = y_t_pad(1:rows,1:columns);
%         y_t_right_bottom_shift = y_t_pad(3:rows+2,3:columns+2);
%         y_t_left_bottom_shift = y_t_pad(3:rows+2,1:columns);
%         y_t_right_top_shift = y_t_pad(1:rows,3:columns+2);
        
      
        %Difference between middle pixel and neighbours
        left_t = abs(y_t(:,:,j,1) - y_t_left_shift);
        right_t = abs(y_t(:,:,j,1) - y_t_right_shift);
        top_t = abs(y_t(:,:,j,1) - y_t_top_shift);
        bottom_t = abs(y_t(:,:,j,1) - y_t_bottom_shift);
        
        left_t1 = abs(y_t(:,:,j,1) - y_t1_left_shift);
        right_t1 = abs(y_t(:,:,j,1) - y_t1_right_shift);
        top_t1 = abs(y_t(:,:,j,1) - y_t1_top_shift);
        bottom_t1 = abs(y_t(:,:,j,1) - y_t1_bottom_shift);
        middle_t1 = abs(y_t(:,:,j,1) - y_t1(:,:,j,1));

        %COMMENTED CODE IS FOR A LARGER NEIGHBORHOOD       
%         right_top_t = abs(y_t(:,:,j,1) - y_t_right_top_shift);
%         left_top_t = abs(y_t(:,:,j,1) - y_t_left_top_shift);
%         right_bottom_t = abs(y_t(:,:,j,1) - y_t_right_bottom_shift);
%         left_bottom_t = abs(y_t(:,:,j,1) - y_t_left_bottom_shift);

%         %Binary masks giving how equal pixels are
%         right_top_t_similar1 = (right_top_t <= threshold1);
%         right_top_t_similar2 = ((right_top_t > threshold1) & (right_top_t <= threshold2));
%         right_top_t_similar3 = ((right_top_t > threshold2) & (right_top_t <= threshold3));
%         right_top_t_not_similar = (right_top_t > threshold3);
%         
%         left_top_t_similar1 = (left_top_t <= threshold1);
%         left_top_t_similar2 = ((left_top_t > threshold1) & (left_top_t <= threshold2));
%         left_top_t_similar3 = ((left_top_t > threshold2) & (left_top_t <= threshold3));
%         left_top_t_not_similar = (left_top_t > threshold3);
%         
%         right_bottom_t_similar1 = (right_bottom_t <= threshold1);
%         right_bottom_t_similar2 = ((right_bottom_t > threshold1) & (right_bottom_t <= threshold2));
%         right_bottom_t_similar3 = ((right_bottom_t > threshold2) & (right_bottom_t <= threshold3));
%         right_bottom_t_not_similar = (right_bottom_t > threshold3);
%         
%         left_bottom_t_similar1 = (left_bottom_t <= threshold1);
%         left_bottom_t_similar2 = ((left_bottom_t > threshold1) & (left_bottom_t <= threshold2));
%         left_bottom_t_similar3 = ((left_bottom_t > threshold2) & (left_bottom_t <= threshold3));
%         left_bottom_t_not_similar = (left_bottom_t > threshold3);
        
        left_t_similar1 = (left_t <= threshold1);
        left_t_similar2 = ((left_t > threshold1) & (left_t <= threshold2));
        left_t_similar3 = ((left_t > threshold2) & (left_t <= threshold3));
        left_t_not_similar = (left_t > threshold3);
        
        left_t1_similar1 = (left_t1 <= threshold1);
        left_t1_similar2 = ((left_t1 > threshold1) & (left_t1 <= threshold2));
        left_t1_similar3 = ((left_t1 > threshold2) & (left_t1 <= threshold3));
        left_t1_not_similar = (left_t1 > threshold3);

        right_t_similar1 = (right_t <= threshold1);
        right_t_similar2 = ((right_t > threshold1) & (right_t <= threshold2));
        right_t_similar3 = ((right_t > threshold2) & (right_t <= threshold3));
        right_t_not_similar = (right_t > threshold3);
        
        right_t1_similar1 = (right_t1 <= threshold1);
        right_t1_similar2 = ((right_t1 > threshold1) & (right_t1 <= threshold2));
        right_t1_similar3 = ((right_t1 > threshold2) & (right_t1 <= threshold3));
        right_t1_not_similar = (right_t1 > threshold3);

        top_t_similar1 = (top_t <= threshold1);
        top_t_similar2 = (top_t > threshold1 & top_t <= threshold2);
        top_t_similar3 = (top_t > threshold2 & top_t <= threshold3);
        top_t_not_similar = (top_t > threshold3);
        
        top_t1_similar1 = (top_t1 <= threshold1);
        top_t1_similar2 = ((top_t1 > threshold1) & (top_t1 <= threshold2));
        top_t1_similar3 = ((top_t1 > threshold2) & (top_t1 <= threshold3));
        top_t1_not_similar = (top_t1 > threshold3);

        bottom_t_similar1 = (bottom_t <= threshold1);
        bottom_t_similar2 = ((bottom_t > threshold1) & (bottom_t <= threshold2));
        bottom_t_similar3 = ((bottom_t > threshold2) & (bottom_t <= threshold3));
        bottom_t_not_similar = (bottom_t > threshold3);
        
        bottom_t1_similar1 = (bottom_t1 <= threshold1);
        bottom_t1_similar2 = ((bottom_t1 > threshold1) & (bottom_t1 <= threshold2));
        bottom_t1_similar3 = ((bottom_t1 > threshold2) & (bottom_t1 <= threshold3));
        bottom_t1_not_similar = (bottom_t1 > threshold3);
        
        middle_t1_similar1 = (middle_t1 <= threshold1);
        middle_t1_similar2 = ((middle_t1 > threshold1) & (middle_t1 <= threshold2));
        middle_t1_similar3 = ((middle_t1 > threshold2) & (middle_t1 <= threshold3));
        middle_t1_not_similar = (middle_t1 > threshold3);
        
%COMMENTED CODE IS FOR A LARGER NEIGHBORHOOD        
        %Total number of neighbour pixels in different similarity classes
        number_of_similar1_px = (left_t_similar1 + right_t_similar1 + top_t_similar1 + bottom_t_similar1 + ...
            left_t1_similar1 + right_t1_similar1 + top_t1_similar1 + bottom_t1_similar1 + middle_t1_similar1);
%             right_top_t_similar1 + left_top_t_similar1 + right_bottom_t_similar1 + left_bottom_t_similar1);
        
        number_of_similar2_px = (left_t_similar2 + right_t_similar2 + top_t_similar2 + bottom_t_similar2 + ...
            left_t1_similar2 + right_t1_similar2 + top_t1_similar2 + bottom_t1_similar2 + middle_t1_similar2 );
%             right_top_t_similar2 + left_top_t_similar2 + right_bottom_t_similar2 + left_bottom_t_similar2);
       
        number_of_similar3_px = (left_t_similar3 + right_t_similar3 + top_t_similar3 + bottom_t_similar3 + ...
            left_t1_similar3 + right_t1_similar3 + top_t1_similar3 + bottom_t1_similar3 + middle_t1_similar3 );
%             right_top_t_similar3 + left_top_t_similar3 + right_bottom_t_similar3 + left_bottom_t_similar3);
        
        number_of_not_similar_px = (left_t_not_similar + right_t_not_similar + top_t_not_similar + bottom_t_not_similar + ...
            left_t1_not_similar + right_t1_not_similar + top_t1_not_similar + bottom_t1_not_similar + middle_t1_not_similar );
%             right_top_t_not_similar + left_top_t_not_similar + right_bottom_t_not_similar + left_bottom_t_not_similar);
          
        %If more thant 8/9 neighbours are not similar, use median filter   
        median_filter_mask = number_of_not_similar_px > 7;
%          median_filter_mask = number_of_not_similar_px > 11;
        
        %Average of all similar pixels, with different weights. Normalizing
        %constant is caclulated for each pixel dependent on how equal
        %neighbours are
        normalizing_constant = ones([rows columns]);
        normalizing_constant = normalizing_constant.*weight_ct_px + weight1.*number_of_similar1_px...
            + weight2.*number_of_similar2_px + weight3.*number_of_similar3_px;

%COMMENTED CODE IS FOR A LARGER NEIGHBORHOOD
        %All the pixels in different similarity classes added together
        similar1 = (y_t_left_shift.*left_t_similar1 + y_t_right_shift.*right_t_similar1 + y_t_top_shift.*top_t_similar1 + y_t_bottom_shift.*bottom_t_similar1 + ...
            + y_t1_left_shift.*left_t1_similar1 + y_t1_right_shift.*right_t1_similar1 + y_t1_top_shift.*top_t1_similar1 + y_t1_bottom_shift.*bottom_t1_similar1 + ...
            + y_t1(:,:,j,1).*middle_t1_similar1);
%             + y_t_right_top_shift.*right_top_t_similar1 + y_t_left_top_shift.*left_top_t_similar1 + y_t_right_bottom_shift.*right_bottom_t_similar1
%             + y_t_left_bottom_shift.*left_bottom_t_similar1 + ...

        
        similar2 = (y_t_left_shift.*left_t_similar2 + y_t_right_shift.*right_t_similar2 + y_t_top_shift.*top_t_similar2 + y_t_bottom_shift.*bottom_t_similar2 + ...
            + y_t1_left_shift.*left_t1_similar2 + y_t1_right_shift.*right_t1_similar2 + y_t1_top_shift.*top_t1_similar2 + y_t1_bottom_shift.*bottom_t1_similar2 + ...
            + y_t1(:,:,j,1).*middle_t1_similar2);
%             + + y_t_right_top_shift.*right_top_t_similar2 + y_t_left_top_shift.*left_top_t_similar2 + y_t_right_bottom_shift.*right_bottom_t_similar2
%             + y_t_left_bottom_shift.*left_bottom_t_similar2 + ...
            

        similar3 = (y_t_left_shift.*left_t_similar3 + y_t_right_shift.*right_t_similar3 + y_t_top_shift.*top_t_similar3 + y_t_bottom_shift.*bottom_t_similar3 + ...
            + y_t1_left_shift.*left_t1_similar3 + y_t1_right_shift.*right_t1_similar3 + y_t1_top_shift.*top_t1_similar3 + y_t1_bottom_shift.*bottom_t1_similar3 + ...
            + y_t1(:,:,j,1).*middle_t1_similar3);
%             + y_t_right_top_shift.*right_top_t_similar3 + y_t_left_top_shift.*left_top_t_similar3 + y_t_right_bottom_shift.*right_bottom_t_similar3 +
%             + y_t_left_bottom_shift.*left_bottom_t_similar3 + ...


        
        %Divide by normalizing constant, and write result
        res(:,:,j,k) = (weight1.*similar1 + weight2.*similar2 + weight3.*similar3 + y_t(:,:,j,1).*weight_ct_px)./normalizing_constant;

        %Overwrite result with median filtered pixels
        res(:,:,j,k) = res(:,:,j,k).*~median_filter_mask + median_filter_mask.*medfilt2(y_t(:,:,j,1),'symmetric');
        
        y_t1(:,:,j,1) = res(:,:,j,k);
        end
        
        
        
    end
    
    
% if input is only one frame, get result from workbench,
% functon must be saved globally with name m00301
else
        
    global result;
    if isempty(result) %check if first frame
        res(:,:,:,1) = in(:,:,:,1);
        
    else
        y_t = in(:,:,:,1);
        
        y_t1 = result(:,:,:,1);
        
        for j = 1:number_of_planes


        %Pad image
        y_t_pad = padarray(y_t(:,:,j,1), [1 1], 'symmetric');
        y_t1_pad = padarray(y_t1(:,:,j,1), [1 1], 'symmetric');
        
        %Matrix with all pixels shifted one position
        y_t_left_shift = y_t_pad(2:rows+1,1:columns);
        y_t_right_shift = y_t_pad(2:rows+1,3:columns+2);
        y_t_top_shift = y_t_pad(1:rows,2:columns+1);
        y_t_bottom_shift = y_t_pad(3:rows+2,2:columns+1);
        
        y_t1_left_shift = y_t1_pad(2:rows+1,1:columns);
        y_t1_right_shift = y_t1_pad(2:rows+1,3:columns+2);
        y_t1_top_shift = y_t1_pad(1:rows,2:columns+1);
        y_t1_bottom_shift = y_t1_pad(3:rows+2,2:columns+1);
        


        
      
        %Difference between middle pixel and neighbours
        left_t = abs(y_t(:,:,j,1) - y_t_left_shift);
        right_t = abs(y_t(:,:,j,1) - y_t_right_shift);
        top_t = abs(y_t(:,:,j,1) - y_t_top_shift);
        bottom_t = abs(y_t(:,:,j,1) - y_t_bottom_shift);
        
        left_t1 = abs(y_t(:,:,j,1) - y_t1_left_shift);
        right_t1 = abs(y_t(:,:,j,1) - y_t1_right_shift);
        top_t1 = abs(y_t(:,:,j,1) - y_t1_top_shift);
        bottom_t1 = abs(y_t(:,:,j,1) - y_t1_bottom_shift);
        middle_t1 = abs(y_t(:,:,j,1) - y_t1(:,:,j,1));
        
%COMMENTED CODE IS FOR A LARGER NEIGHBORHOOD
%corner pixels
%         y_t_left_top_shift = y_t_pad(1:rows,1:columns);
%         y_t_right_bottom_shift = y_t_pad(3:rows+2,3:columns+2);
%         y_t_left_bottom_shift = y_t_pad(3:rows+2,1:columns);
%         y_t_right_top_shift = y_t_pad(1:rows,3:columns+2);
%         
%         right_top_t = abs(y_t(:,:,j,1) - y_t_right_top_shift);
%         left_top_t = abs(y_t(:,:,j,1) - y_t_left_top_shift);
%         right_bottom_t = abs(y_t(:,:,j,1) - y_t_right_bottom_shift);
%         left_bottom_t = abs(y_t(:,:,j,1) - y_t_left_bottom_shift);
% 
%         %Binary masks giving how equal pixels are
%         right_top_t_similar1 = (right_top_t <= threshold1);
%         right_top_t_similar2 = ((right_top_t > threshold1) & (right_top_t <= threshold2));
%         right_top_t_similar3 = ((right_top_t > threshold2) & (right_top_t <= threshold3));
%         right_top_t_not_similar = (right_top_t > threshold3);
%         
%         left_top_t_similar1 = (left_top_t <= threshold1);
%         left_top_t_similar2 = ((left_top_t > threshold1) & (left_top_t <= threshold2));
%         left_top_t_similar3 = ((left_top_t > threshold2) & (left_top_t <= threshold3));
%         left_top_t_not_similar = (left_top_t > threshold3);
%         
%         right_bottom_t_similar1 = (right_bottom_t <= threshold1);
%         right_bottom_t_similar2 = ((right_bottom_t > threshold1) & (right_bottom_t <= threshold2));
%         right_bottom_t_similar3 = ((right_bottom_t > threshold2) & (right_bottom_t <= threshold3));
%         right_bottom_t_not_similar = (right_bottom_t > threshold3);
%         
%         left_bottom_t_similar1 = (left_bottom_t <= threshold1);
%         left_bottom_t_similar2 = ((left_bottom_t > threshold1) & (left_bottom_t <= threshold2));
%         left_bottom_t_similar3 = ((left_bottom_t > threshold2) & (left_bottom_t <= threshold3));
%         left_bottom_t_not_similar = (left_bottom_t > threshold3);
%

        left_t_similar1 = (left_t <= threshold1);
        left_t_similar2 = ((left_t > threshold1) & (left_t <= threshold2));
        left_t_similar3 = ((left_t > threshold2) & (left_t <= threshold3));
        left_t_not_similar = (left_t > threshold3);
        
        left_t1_similar1 = (left_t1 <= threshold1);
        left_t1_similar2 = ((left_t1 > threshold1) & (left_t1 <= threshold2));
        left_t1_similar3 = ((left_t1 > threshold2) & (left_t1 <= threshold3));
        left_t1_not_similar = (left_t1 > threshold3);

        right_t_similar1 = (right_t <= threshold1);
        right_t_similar2 = ((right_t > threshold1) & (right_t <= threshold2));
        right_t_similar3 = ((right_t > threshold2) & (right_t <= threshold3));
        right_t_not_similar = (right_t > threshold3);
        
        right_t1_similar1 = (right_t1 <= threshold1);
        right_t1_similar2 = ((right_t1 > threshold1) & (right_t1 <= threshold2));
        right_t1_similar3 = ((right_t1 > threshold2) & (right_t1 <= threshold3));
        right_t1_not_similar = (right_t1 > threshold3);

        top_t_similar1 = (top_t <= threshold1);
        top_t_similar2 = (top_t > threshold1 & top_t <= threshold2);
        top_t_similar3 = (top_t > threshold2 & top_t <= threshold3);
        top_t_not_similar = (top_t > threshold3);
        
        top_t1_similar1 = (top_t1 <= threshold1);
        top_t1_similar2 = ((top_t1 > threshold1) & (top_t1 <= threshold2));
        top_t1_similar3 = ((top_t1 > threshold2) & (top_t1 <= threshold3));
        top_t1_not_similar = (top_t1 > threshold3);

        bottom_t_similar1 = (bottom_t <= threshold1);
        bottom_t_similar2 = ((bottom_t > threshold1) & (bottom_t <= threshold2));
        bottom_t_similar3 = ((bottom_t > threshold2) & (bottom_t <= threshold3));
        bottom_t_not_similar = (bottom_t > threshold3);
        
        bottom_t1_similar1 = (bottom_t1 <= threshold1);
        bottom_t1_similar2 = ((bottom_t1 > threshold1) & (bottom_t1 <= threshold2));
        bottom_t1_similar3 = ((bottom_t1 > threshold2) & (bottom_t1 <= threshold3));
        bottom_t1_not_similar = (bottom_t1 > threshold3);
        
        middle_t1_similar1 = (middle_t1 <= threshold1);
        middle_t1_similar2 = ((middle_t1 > threshold1) & (middle_t1 <= threshold2));
        middle_t1_similar3 = ((middle_t1 > threshold2) & (middle_t1 <= threshold3));
        middle_t1_not_similar = (middle_t1 > threshold3);
        
%COMMENTED CODE IS FOR A LARGER NEIGHBORHOOD        
        %Total number of neighbour pixels in different similarity classes
        number_of_similar1_px = (left_t_similar1 + right_t_similar1 + top_t_similar1 + bottom_t_similar1 + ...
            left_t1_similar1 + right_t1_similar1 + top_t1_similar1 + bottom_t1_similar1 + middle_t1_similar1);
%                     right_top_t_similar1 + left_top_t_similar1 + right_bottom_t_similar1 + left_bottom_t_similar1);%%%%%%%
        
        number_of_similar2_px = (left_t_similar2 + right_t_similar2 + top_t_similar2 + bottom_t_similar2 + ...
            left_t1_similar2 + right_t1_similar2 + top_t1_similar2 + bottom_t1_similar2 + middle_t1_similar2);        
%             right_top_t_similar2 + left_top_t_similar2 + right_bottom_t_similar2 + left_bottom_t_similar2);%%%%%%%%%%
       
        number_of_similar3_px = (left_t_similar3 + right_t_similar3 + top_t_similar3 + bottom_t_similar3 + ...
            left_t1_similar3 + right_t1_similar3 + top_t1_similar3 + bottom_t1_similar3 + middle_t1_similar3);
%             right_top_t_similar3 + left_top_t_similar3 + right_bottom_t_similar3 + left_bottom_t_similar3);%%%%%%%%%%
        
        number_of_not_similar_px = (left_t_not_similar + right_t_not_similar + top_t_not_similar + bottom_t_not_similar + ...
            left_t1_not_similar + right_t1_not_similar + top_t1_not_similar + bottom_t1_not_similar + middle_t1_not_similar);
%             right_top_t_not_similar + left_top_t_not_similar + right_bottom_t_not_similar + left_bottom_t_not_similar);        %%%%%%%%%%
          
        %If more thant 8/9 neighbours are not similar, use median filter   
        median_filter_mask = number_of_not_similar_px > 7;
        %%%%%%%%%%%%
       %  median_filter_mask = number_of_not_similar_px > 11;
        
        %Average of all similar pixels, with different weights. Normalizing
        %constant is caclulated for each pixel dependent on how equal
        %neighbours are
        normalizing_constant = ones([rows columns]);
        normalizing_constant = normalizing_constant.*weight_ct_px + weight1.*number_of_similar1_px...
            + weight2.*number_of_similar2_px + weight3.*number_of_similar3_px;

%COMMENTED CODE IS FOR A LARGER NEIGHBORHOOD
        %All the pixels in different similarity classes added together
        similar1 = (y_t_left_shift.*left_t_similar1 + y_t_right_shift.*right_t_similar1 + y_t_top_shift.*top_t_similar1 + y_t_bottom_shift.*bottom_t_similar1 + ...
            + y_t1_left_shift.*left_t1_similar1 + y_t1_right_shift.*right_t1_similar1 + y_t1_top_shift.*top_t1_similar1 + y_t1_bottom_shift.*bottom_t1_similar1 + ...
            + y_t1(:,:,j,1).*middle_t1_similar1);
          %  + y_t_right_top_shift.*right_top_t_similar1 + y_t_left_top_shift.*left_top_t_similar1 + y_t_right_bottom_shift.*right_bottom_t_similar1 + ...  %%%%%%%%%%
          %  + y_t_left_bottom_shift.*left_bottom_t_similar1);     %%%%%%%%%%

        
        similar2 = (y_t_left_shift.*left_t_similar2 + y_t_right_shift.*right_t_similar2 + y_t_top_shift.*top_t_similar2 + y_t_bottom_shift.*bottom_t_similar2 + ...
            + y_t1_left_shift.*left_t1_similar2 + y_t1_right_shift.*right_t1_similar2 + y_t1_top_shift.*top_t1_similar2 + y_t1_bottom_shift.*bottom_t1_similar2 + ...
            + y_t1(:,:,j,1).*middle_t1_similar2);
          %  + + y_t_right_top_shift.*right_top_t_similar2 + y_t_left_top_shift.*left_top_t_similar2 + y_t_right_bottom_shift.*right_bottom_t_similar2+ ...%%%%%%%%%%
           % + y_t_left_bottom_shift.*left_bottom_t_similar2);%%%%%%%%%%
            

        similar3 = (y_t_left_shift.*left_t_similar3 + y_t_right_shift.*right_t_similar3 + y_t_top_shift.*top_t_similar3 + y_t_bottom_shift.*bottom_t_similar3 + ...
            + y_t1_left_shift.*left_t1_similar3 + y_t1_right_shift.*right_t1_similar3 + y_t1_top_shift.*top_t1_similar3 + y_t1_bottom_shift.*bottom_t1_similar3 + ...
            + y_t1(:,:,j,1).*middle_t1_similar3);
            
        %+ y_t_right_top_shift.*right_top_t_similar3 + y_t_left_top_shift.*left_top_t_similar3 + y_t_right_bottom_shift.*right_bottom_t_similar3 + ...%%%%%%%%%%
       %     + y_t_left_bottom_shift.*left_bottom_t_similar3);%%%%%%%%%%


        
        %Divide by normalizing constant, and write result
        res(:,:,j,1) = (weight1.*similar1 + weight2.*similar2 + weight3.*similar3 + y_t(:,:,j,1).*weight_ct_px)./normalizing_constant;

        %Overwrite result with median filtered pixels
        res(:,:,j,1) = res(:,:,j,1).*~median_filter_mask + median_filter_mask.*medfilt2(y_t(:,:,j,1),'symmetric');

        y_t1(:,:,j,1) = res(:,:,j,1);
       
        end
        
    end
end


frame_runtime = cputime-t






